<?php
/**
* @package		PayPlans
* @copyright	Copyright (C) 2010 - 2017 Stack Ideas Sdn Bhd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* PayPlans is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/
defined('_JEXEC') or die('Unauthorized Access');

class PayplansAppRemita extends PayplansAppPayment
{
	protected $_location	= __FILE__;

	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
   	{
		if (is_object($data)) {
			$data = (array)$data;
		}

		$root = JURI::root();
		if (XiFactory::getConfig()->https == true) {
		    $root = JString::str_ireplace("http:", "https:", $root);
		}

		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$merchantId = $this->getAppParam('merchant_id',false);
		$serviceId = $this->getAppParam('service_type_id', false);
		$apiKey = $this->getAppParam('api_key',false);
		$orderId = $payment->getKey();
		$amt = $invoice->getTotal();
		$responseurl = $root.'index.php?option=com_payplans&gateway=remita&view=payment&task=complete&action=success&payment_key='.$orderId; 

		// generate request signature
		$concatString = $merchantId . $serviceId . $orderId . $amt . $responseurl . $apiKey;
		$hash = hash('sha512', $concatString);

		if ($this->getAppParam('sandbox',0)) {
			$post_url = 'http://www.remitademo.net/remita/ecomm/init.reg';
		} else {
			$post_url = 'http://www.remita.net/remita/ecomm/init.reg';
		}
	    	
	    	$this->assign('merchantId', $merchantId);
	    	$this->assign('serviceTypeId', $serviceId);
	    	$this->assign('amt', $amt);
	    	$this->assign('responseurl', $responseurl);
	    	$this->assign('hash', $hash);
	    	$this->assign('orderId', $orderId);
	    	$this->assign('payment', $payment);
	    	$this->assign('url', $post_url); 
	    	
		return $this->_render('postform');
  	}

    	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if ($action == 'cancel') {
			return true;
		}

		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount = number_format($invoice->getTotal(),2);
		
		$this->_processPayment($invoice, $payment, $data);
	
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	
	public function _processPayment($invoice, $payment, $data)
	{
		$error = '';
		if (isset( $data['orderID'] )) { 
			$merchantId = $this->getAppParam('merchant_id',false);
			$serviceId = $this->getAppParam('service_type_id', false);
			$apiKey = $this->getAppParam('api_key',false);
			$orderId = $data['orderID'];

			if ($this->getAppParam('sandbox',0)) {
				$statusUrl      = "http://www.remitademo.net/remita/ecomm";
			} else {
				$statusUrl      = "http://www.remita.net/remita/ecomm";
			}

			$concatString = $orderId . $apiKey . $merchantId;
			$hash = hash('sha512', $concatString);
			$url = $statusUrl . '/' . $merchantId  . '/' . $orderId . '/' . $hash . '/' . 'orderstatus.reg';
		
			//  Initiate curl
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_URL,$url);
			$result	= curl_exec($ch);
			curl_close($ch);
			$response 		= json_decode($result, true);

			$response_code 	= $response['status'];

			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $payment->getBuyer())
				    ->set('invoice_id', $invoice->getId())
				    ->set('payment_id', $payment->getId())
				    ->set('gateway_txn_id', isset($response['RRR']) 		? $response['returned_transref'] 			: 0)
				    ->set('gateway_subscr_id', 	0)
				    ->set('gateway_parent_txn', 0)
				    ->set('params', PayplansHelperParam::arrayToIni($response));

			if ($response_code == '01' || $response_code == '00') {
				$transaction->set('amount', $response['amount'])
					    ->set('message', JText::_('COM_PAYPLANS_PAYMENT_APP_REMITA_TRANSACTION_COMPLETED'));
			} else {
				$transaction->set('message', JText::_('COM_PAYPLANS_PAYMENT_APP_REMITA_TRANSACTION_FAILED'));
			}

			 //store the response in the payment AND save the payment
			if (!$transaction->save()) {
				$message = JText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			}

		} else {
			$error  = "Order Id Missing";
		}
		
		if ($error) {
			$message = JText::_('COM_PAYPLANS_LOGGER_ERROR_IN_REMITA_PAYMENT_PROCESS');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
		
		$payment->save();
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
}
